/* Java Polyhedron interface code.
   Copyright (C) 2001-2009 Roberto Bagnara <bagnara@cs.unipr.it>

This file is part of the Parma Polyhedra Library (PPL).

The PPL is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

The PPL is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1307, USA.

For the most up-to-date information see the Parma Polyhedra Library
site: http://www.cs.unipr.it/ppl/ . */

#include "ppl_java_common.defs.hh"
#include "parma_polyhedra_library_Polyhedron.h"
#include "parma_polyhedra_library_C_Polyhedron.h"
#include "parma_polyhedra_library_NNC_Polyhedron.h"
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::Interfaces::Java;

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_finalize
(JNIEnv* env, jobject j_this) try {
  if (!is_java_marked(env, j_this)) {
    C_Polyhedron* this_ptr
      = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_this));
    delete this_ptr;
  }
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_finalize
(JNIEnv* env, jobject j_this) try {
  if (!is_java_marked(env, j_this)) {
    NNC_Polyhedron* this_ptr
      = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_this));
    delete this_ptr;
  }
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__JLparma_1polyhedra_1library_Degenerate_1Element_2
(JNIEnv* env, jobject j_this, jlong j_dim, jobject j_degenerate_element) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(j_dim);
  jclass degenerate_element_class
    = env->FindClass("parma_polyhedra_library/Degenerate_Element");
  jmethodID degenerate_element_ordinal_id
    = env->GetMethodID(degenerate_element_class, "ordinal", "()I");
  jint j_degenerate_element_int
    = env->CallIntMethod(j_degenerate_element, degenerate_element_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_degenerate_element_int) {
  case 0:
    this_ptr = new C_Polyhedron(ppl_dim, UNIVERSE);
    break;
  case 1:
    this_ptr = new C_Polyhedron(ppl_dim, EMPTY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__JLparma_1polyhedra_1library_Degenerate_1Element_2
(JNIEnv* env, jobject j_this, jlong j_dim, jobject j_degenerate_element) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(j_dim);
  jclass degenerate_element_class
    = env->FindClass("parma_polyhedra_library/Degenerate_Element");
  jmethodID degenerate_element_ordinal_id
    = env->GetMethodID(degenerate_element_class, "ordinal", "()I");
  jint j_degenerate_element_int
    = env->CallIntMethod(j_degenerate_element, degenerate_element_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_degenerate_element_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(ppl_dim, UNIVERSE);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(ppl_dim, EMPTY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_C_1Polyhedron_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  C_Polyhedron* y_ptr
    = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_C_1Polyhedron_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  C_Polyhedron* y_ptr
    = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_NNC_1Polyhedron_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  NNC_Polyhedron* y_ptr
    = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_NNC_1Polyhedron_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  NNC_Polyhedron* y_ptr
    = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Grid_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Grid_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Rational_1Box_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Rational_Box* y_ptr
    = reinterpret_cast<Rational_Box*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Rational_1Box_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Rational_Box* y_ptr
    = reinterpret_cast<Rational_Box*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpz_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  BD_Shape<mpz_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpz_class>*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpz_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  BD_Shape<mpz_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpz_class>*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpq_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  BD_Shape<mpq_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpq_class>*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpq_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  BD_Shape<mpq_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpq_class>*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpz_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Octagonal_Shape<mpz_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpz_class>*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpz_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Octagonal_Shape<mpz_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpz_class>*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpq_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Octagonal_Shape<mpq_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpq_class>*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpq_1class_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Octagonal_Shape<mpq_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpq_class>*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Double_1Box_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Double_Box* y_ptr
    = reinterpret_cast<Double_Box*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Double_1Box_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Double_Box* y_ptr
    = reinterpret_cast<Double_Box*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1double_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  BD_Shape<double>* y_ptr
    = reinterpret_cast<BD_Shape<double>*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1double_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  BD_Shape<double>* y_ptr
    = reinterpret_cast<BD_Shape<double>*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1double_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Octagonal_Shape<double>* y_ptr
    = reinterpret_cast<Octagonal_Shape<double>*>(get_ptr(env, j_y));
  C_Polyhedron* this_ptr = new C_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1double_2
(JNIEnv* env, jobject  j_this, jobject j_y) try {
  Octagonal_Shape<double>* y_ptr
    = reinterpret_cast<Octagonal_Shape<double>*>(get_ptr(env, j_y));
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(*y_ptr);
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL







JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_C_1Polyhedron_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  C_Polyhedron* y_ptr
    = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_C_1Polyhedron_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  C_Polyhedron* y_ptr
    = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_NNC_1Polyhedron_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  NNC_Polyhedron* y_ptr
    = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_NNC_1Polyhedron_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  NNC_Polyhedron* y_ptr
    = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Grid_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Grid_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Grid* y_ptr
    = reinterpret_cast<Grid*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Rational_1Box_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Rational_Box* y_ptr
    = reinterpret_cast<Rational_Box*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Rational_1Box_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Rational_Box* y_ptr
    = reinterpret_cast<Rational_Box*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpz_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  BD_Shape<mpz_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpz_class>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpz_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  BD_Shape<mpz_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpz_class>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpq_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  BD_Shape<mpq_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpq_class>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1mpq_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  BD_Shape<mpq_class>* y_ptr
    = reinterpret_cast<BD_Shape<mpq_class>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpz_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Octagonal_Shape<mpz_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpz_class>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpz_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Octagonal_Shape<mpz_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpz_class>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpq_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Octagonal_Shape<mpq_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpq_class>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1mpq_1class_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Octagonal_Shape<mpq_class>* y_ptr
    = reinterpret_cast<Octagonal_Shape<mpq_class>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Double_1Box_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Double_Box* y_ptr
    = reinterpret_cast<Double_Box*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Double_1Box_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Double_Box* y_ptr
    = reinterpret_cast<Double_Box*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1double_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  BD_Shape<double>* y_ptr
    = reinterpret_cast<BD_Shape<double>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_BD_1Shape_1double_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  BD_Shape<double>* y_ptr
    = reinterpret_cast<BD_Shape<double>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1double_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Octagonal_Shape<double>* y_ptr
    = reinterpret_cast<Octagonal_Shape<double>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  C_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new C_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new C_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new C_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Octagonal_1Shape_1double_2Lparma_1polyhedra_1library_Complexity_1Class_2
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_complexity) try {
  Octagonal_Shape<double>* y_ptr
    = reinterpret_cast<Octagonal_Shape<double>*>(get_ptr(env, j_y));
  jclass complexity_class
    = env->FindClass("parma_polyhedra_library/Complexity_Class");
  jmethodID complexity_ordinal_id
    = env->GetMethodID(complexity_class, "ordinal", "()I");
  jint j_complexity_int
    = env->CallIntMethod(j_complexity, complexity_ordinal_id);
  NNC_Polyhedron* this_ptr;
  switch (j_complexity_int) {
  case 0:
    this_ptr = new NNC_Polyhedron(*y_ptr, POLYNOMIAL_COMPLEXITY);
    break;
  case 1:
    this_ptr = new NNC_Polyhedron(*y_ptr, SIMPLEX_COMPLEXITY);
    break;
  case 2:
    this_ptr = new NNC_Polyhedron(*y_ptr, ANY_COMPLEXITY);
    break;
  default:
    throw std::runtime_error("PPL Java interface internal error");
  }
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL







JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Constraint_1System_2
(JNIEnv* env, jobject j_this, jobject j_iterable) try {
  Constraint_System cs
    = build_cxx_constraint_system(env, j_iterable);
  C_Polyhedron* this_ptr = new C_Polyhedron(cs, Recycle_Input());
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Constraint_1System_2
(JNIEnv* env, jobject j_this, jobject j_iterable) try {
  Constraint_System cs
    = build_cxx_constraint_system(env, j_iterable);
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(cs, Recycle_Input());
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Congruence_1System_2
(JNIEnv* env, jobject j_this, jobject j_iterable) try {
  Congruence_System cs
    = build_cxx_congruence_system(env, j_iterable);
  C_Polyhedron* this_ptr = new C_Polyhedron(cs, Recycle_Input());
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Congruence_1System_2
(JNIEnv* env, jobject j_this, jobject j_iterable) try {
  Congruence_System cs
    = build_cxx_congruence_system(env, j_iterable);
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(cs, Recycle_Input());
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Generator_1System_2
(JNIEnv* env, jobject j_this, jobject j_iterable) try {
  Generator_System cs
    = build_cxx_generator_system(env, j_iterable);
  C_Polyhedron* this_ptr = new C_Polyhedron(cs, Recycle_Input());
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_build_1cpp_1object__Lparma_1polyhedra_1library_Generator_1System_2
(JNIEnv* env, jobject j_this, jobject j_iterable) try {
  Generator_System cs
    = build_cxx_generator_system(env, j_iterable);
  NNC_Polyhedron* this_ptr = new NNC_Polyhedron(cs, Recycle_Input());
  set_ptr(env, j_this, this_ptr);
}
CATCH_ALL








JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_swap
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  this_ptr->swap(*y_ptr);
}
CATCH_ALL





JNIEXPORT jlong JNICALL
Java_parma_1polyhedra_1library_Polyhedron_space_1dimension
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->space_dimension();
  }
  CATCH_ALL;
  return 0;
}

JNIEXPORT jlong JNICALL
Java_parma_1polyhedra_1library_Polyhedron_affine_1dimension
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->affine_dimension();
  }
  CATCH_ALL;
  return 0;
}






JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Polyhedron_relation_1with__Lparma_1polyhedra_1library_Constraint_2
(JNIEnv* env, jobject j_this, jobject j_c) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Constraint c = build_cxx_constraint(env, j_c);
    Poly_Con_Relation pcr = this_ptr->relation_with(c);
    return build_java_poly_con_relation(env, pcr);
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Polyhedron_relation_1with__Lparma_1polyhedra_1library_Generator_2
(JNIEnv* env, jobject j_this, jobject j_c) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Generator c = build_cxx_generator(env, j_c);
    Poly_Gen_Relation pcr = this_ptr->relation_with(c);
    return build_java_poly_gen_relation(env, pcr);
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Polyhedron_relation_1with__Lparma_1polyhedra_1library_Congruence_2
(JNIEnv* env, jobject j_this, jobject j_c) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Congruence c = build_cxx_congruence(env, j_c);
    Poly_Con_Relation pcr = this_ptr->relation_with(c);
    return build_java_poly_con_relation(env, pcr);
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}






JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Polyhedron_constraints
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return build_java_constraint_system(env, this_ptr->constraints());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Polyhedron_congruences
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return build_java_congruence_system(env, this_ptr->congruences());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Polyhedron_generators
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return build_java_generator_system(env, this_ptr->generators());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}






JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Polyhedron_minimized_1constraints
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return build_java_constraint_system(env,
             this_ptr->minimized_constraints());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Polyhedron_minimized_1congruences
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return build_java_congruence_system(env,
             this_ptr->minimized_congruences());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_Polyhedron_minimized_1generators
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return build_java_generator_system(env,
             this_ptr->minimized_generators());
  }
  CATCH_ALL;
  jobject null = 0;
  return null;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_is_1empty
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->is_empty();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_is_1universe
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->is_universe();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_is_1bounded
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->is_bounded();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_contains_1integer_1point
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->contains_integer_point();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_is_1topologically_1closed
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->is_topologically_closed();
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_is_1discrete
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->is_discrete();
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_topological_1closure_1assign
(JNIEnv* env, jobject j_this) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  this_ptr->topological_closure_assign();
}
CATCH_ALL






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_bounds_1from_1above
(JNIEnv* env, jobject j_this, jobject j_le) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    return this_ptr->bounds_from_above(le);
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_bounds_1from_1below
(JNIEnv* env, jobject j_this, jobject j_le) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    return this_ptr->bounds_from_below(le);
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_maximize__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_By_1Reference_2
(JNIEnv* env, jobject j_this, jobject j_le,
 jobject j_num, jobject j_den, jobject j_ref_boolean) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    PPL_DIRTY_TEMP_COEFFICIENT(num);
    PPL_DIRTY_TEMP_COEFFICIENT(den);
    num = build_cxx_coeff(env, j_num);
    den = build_cxx_coeff(env, j_den);
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    bool b_value;
    if (this_ptr->maximize(le, num, den, b_value)) {
      jobject j_num_result = build_java_coeff(env, num);
      jobject j_den_result = build_java_coeff(env, den);
      set_coefficient(env, j_num, j_num_result);
      set_coefficient(env, j_den, j_den_result);
      jobject j_boolean = bool_to_j_boolean(env, b_value);
      set_by_reference(env, j_ref_boolean, j_boolean);
      return true;
    }
    return false;
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_minimize__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_By_1Reference_2
(JNIEnv* env, jobject j_this, jobject j_le,
 jobject j_num, jobject j_den, jobject j_ref_boolean) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    PPL_DIRTY_TEMP_COEFFICIENT(num);
    PPL_DIRTY_TEMP_COEFFICIENT(den);
    num = build_cxx_coeff(env, j_num);
    den = build_cxx_coeff(env, j_den);
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    bool b_value;
    if (this_ptr->minimize(le, num, den, b_value)) {
      jobject j_num_result = build_java_coeff(env, num);
      jobject j_den_result = build_java_coeff(env, den);
      set_coefficient(env, j_num, j_num_result);
      set_coefficient(env, j_den, j_den_result);
      jobject j_boolean = bool_to_j_boolean(env, b_value);
      set_by_reference(env, j_ref_boolean, j_boolean);
      return true;
    }
    return false;
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_maximize__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_By_1Reference_2Lparma_1polyhedra_1library_Generator_2
(JNIEnv* env, jobject j_this, jobject j_le,
 jobject j_num, jobject j_den, jobject j_ref_boolean, jobject j_g) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    PPL_DIRTY_TEMP_COEFFICIENT(num);
    PPL_DIRTY_TEMP_COEFFICIENT(den);
    num = build_cxx_coeff(env, j_num);
    den = build_cxx_coeff(env, j_den);
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    bool b_value;
    Generator g = point();
    if (this_ptr->maximize(le, num, den, b_value, g)) {
      jobject j_num_result = build_java_coeff(env, num);
      jobject j_den_result = build_java_coeff(env, den);
      jobject j_g_result = build_java_generator(env, g);
      set_coefficient(env, j_num, j_num_result);
      set_coefficient(env, j_den, j_den_result);
      jobject j_boolean = bool_to_j_boolean(env, b_value);
      set_by_reference(env, j_ref_boolean, j_boolean);
      set_generator(env, j_g, j_g_result);
      return true;
    }
    return false;
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_minimize__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_Coefficient_2Lparma_1polyhedra_1library_By_1Reference_2Lparma_1polyhedra_1library_Generator_2
(JNIEnv* env, jobject j_this, jobject j_le,
 jobject j_num, jobject j_den, jobject j_ref_boolean, jobject j_g) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    PPL_DIRTY_TEMP_COEFFICIENT(num);
    PPL_DIRTY_TEMP_COEFFICIENT(den);
    num = build_cxx_coeff(env, j_num);
    den = build_cxx_coeff(env, j_den);
    Linear_Expression le = build_cxx_linear_expression(env, j_le);
    bool b_value;
    Generator g = point();
    if (this_ptr->minimize(le, num, den, b_value, g)) {
      jobject j_num_result = build_java_coeff(env, num);
      jobject j_den_result = build_java_coeff(env, den);
      jobject j_g_result = build_java_generator(env, g);
      set_coefficient(env, j_num, j_num_result);
      set_coefficient(env, j_den, j_den_result);
      jobject j_boolean = bool_to_j_boolean(env, b_value);
      set_by_reference(env, j_ref_boolean, j_boolean);
      set_generator(env, j_g, j_g_result);
      return true;
    }
    return false;
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_contains
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Polyhedron* y_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
    return this_ptr->contains(*y_ptr);
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_strictly_1contains
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Polyhedron* y_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
    return this_ptr->strictly_contains(*y_ptr);
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_is_1disjoint_1from
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Polyhedron* y_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
    return this_ptr->is_disjoint_from(*y_ptr);
  }
  CATCH_ALL;
  return false;
}






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_equals
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Polyhedron* y_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
    return *this_ptr == *y_ptr;
  }
  CATCH_ALL;
  return false;
}





JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_OK
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->OK();
  }
  CATCH_ALL;
  return false;
}





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_add_1constraint
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Constraint c = build_cxx_constraint(env, j_c);
  this_ptr->add_constraint(c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_add_1congruence
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Congruence c = build_cxx_congruence(env, j_c);
  this_ptr->add_congruence(c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_add_1generator
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Generator c = build_cxx_generator(env, j_c);
  this_ptr->add_generator(c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_add_1constraints
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Constraint_System cs = build_cxx_constraint_system(env, j_cs);
  this_ptr->add_constraints(cs);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_add_1congruences
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Congruence_System cs = build_cxx_congruence_system(env, j_cs);
  this_ptr->add_congruences(cs);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_add_1generators
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Generator_System cs = build_cxx_generator_system(env, j_cs);
  this_ptr->add_generators(cs);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_refine_1with_1constraint
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Constraint c = build_cxx_constraint(env, j_c);
  this_ptr->refine_with_constraint(c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_refine_1with_1congruence
(JNIEnv* env, jobject j_this, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Congruence c = build_cxx_congruence(env, j_c);
  this_ptr->refine_with_congruence(c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_refine_1with_1constraints
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Constraint_System cs
    = build_cxx_constraint_system(env, j_cs);
  this_ptr->refine_with_constraints(cs);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_refine_1with_1congruences
(JNIEnv* env, jobject j_this, jobject j_cs) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Congruence_System cs
    = build_cxx_congruence_system(env, j_cs);
  this_ptr->refine_with_congruences(cs);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_intersection_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  this_ptr->intersection_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_upper_1bound_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  this_ptr->upper_bound_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_difference_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  this_ptr->difference_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_concatenate_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  this_ptr->concatenate_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_time_1elapse_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  this_ptr->time_elapse_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_poly_1hull_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  this_ptr->poly_hull_assign(*y_ptr);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_poly_1difference_1assign
(JNIEnv* env, jobject j_this, jobject j_y) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  this_ptr->poly_difference_assign(*y_ptr);
}
CATCH_ALL






JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_upper_1bound_1assign_1if_1exact
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    C_Polyhedron* this_ptr
      = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_this));
    C_Polyhedron* y_ptr
      = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_y));
    return this_ptr->upper_bound_assign_if_exact(*y_ptr);
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_upper_1bound_1assign_1if_1exact
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    NNC_Polyhedron* this_ptr
      = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_this));
    NNC_Polyhedron* y_ptr
      = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_y));
    return this_ptr->upper_bound_assign_if_exact(*y_ptr);
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_poly_1hull_1assign_1if_1exact
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    C_Polyhedron* this_ptr
      = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_this));
    C_Polyhedron* y_ptr
      = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_y));
    return this_ptr->poly_hull_assign_if_exact(*y_ptr);
  }
  CATCH_ALL;
  return false;
}

JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_poly_1hull_1assign_1if_1exact
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    NNC_Polyhedron* this_ptr
      = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_this));
    NNC_Polyhedron* y_ptr
      = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_y));
    return this_ptr->poly_hull_assign_if_exact(*y_ptr);
  }
  CATCH_ALL;
  return false;
}







JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_simplify_1using_1context_1assign
(JNIEnv* env, jobject j_this, jobject j_y) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Polyhedron* y_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
    return this_ptr->simplify_using_context_assign(*y_ptr);
  }
  CATCH_ALL;
  return false;
}





JNIEXPORT jboolean JNICALL
Java_parma_1polyhedra_1library_Polyhedron_constrains
(JNIEnv* env, jobject j_this, jobject j_var) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    Variable v = build_cxx_variable(env, j_var);
    return this_ptr->constrains(v);
  }
  CATCH_ALL;
  return false;
}





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_unconstrain_1space_1dimension
(JNIEnv* env, jobject j_this, jobject j_var) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_var);
  this_ptr->unconstrain(v);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_unconstrain_1space_1dimensions
(JNIEnv* env, jobject j_this, jobject j_v_set) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variables_Set v_set = build_cxx_variables_set(env, j_v_set);
  this_ptr->unconstrain(v_set);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_affine_1image
(JNIEnv* env, jobject j_this, jobject j_v, jobject j_le, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->affine_image(v, le, c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_affine_1preimage
(JNIEnv* env, jobject j_this, jobject j_v, jobject j_le, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->affine_preimage(v, le, c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_bounded_1affine_1image
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_lhs, jobject j_rhs, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->bounded_affine_image(v, lhs, rhs, c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_bounded_1affine_1preimage
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_lhs, jobject j_rhs, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->bounded_affine_preimage(v, lhs, rhs, c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_generalized_1affine_1image__Lparma_1polyhedra_1library_Variable_2Lparma_1polyhedra_1library_Relation_1Symbol_2Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_relsym, jobject j_le, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->generalized_affine_image(v, relsym, le, c);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_generalized_1affine_1preimage__Lparma_1polyhedra_1library_Variable_2Lparma_1polyhedra_1library_Relation_1Symbol_2Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Coefficient_2
(JNIEnv* env, jobject j_this,
 jobject j_v, jobject j_relsym, jobject j_le, jobject j_c) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_v);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  Linear_Expression le = build_cxx_linear_expression(env, j_le);
  PPL_DIRTY_TEMP_COEFFICIENT(c);
  c = build_cxx_coeff(env, j_c);
  this_ptr->generalized_affine_preimage(v, relsym, le, c);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_generalized_1affine_1image__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Relation_1Symbol_2Lparma_1polyhedra_1library_Linear_1Expression_2
(JNIEnv* env, jobject j_this,
 jobject j_lhs, jobject j_relsym, jobject j_rhs) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  this_ptr->generalized_affine_image(lhs, relsym, rhs);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_generalized_1affine_1preimage__Lparma_1polyhedra_1library_Linear_1Expression_2Lparma_1polyhedra_1library_Relation_1Symbol_2Lparma_1polyhedra_1library_Linear_1Expression_2
(JNIEnv* env, jobject j_this,
 jobject j_lhs, jobject j_relsym, jobject j_rhs) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Linear_Expression lhs = build_cxx_linear_expression(env, j_lhs);
  Linear_Expression rhs = build_cxx_linear_expression(env, j_rhs);
  Relation_Symbol relsym = build_cxx_relsym(env, j_relsym);
  this_ptr->generalized_affine_preimage(lhs, relsym, rhs);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_add_1space_1dimensions_1and_1embed
(JNIEnv* env, jobject j_this, jlong dim) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(dim);
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  this_ptr->add_space_dimensions_and_embed(ppl_dim);
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_add_1space_1dimensions_1and_1project
(JNIEnv* env, jobject j_this, jlong dim) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(dim);
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  this_ptr->add_space_dimensions_and_project(ppl_dim);
}
CATCH_ALL






JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_remove_1space_1dimensions
(JNIEnv* env, jobject j_this, jobject j_v_set) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variables_Set v_set = build_cxx_variables_set(env, j_v_set);
  this_ptr->remove_space_dimensions(v_set);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_remove_1higher_1space_1dimensions
(JNIEnv* env, jobject j_this, jlong dim) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(dim);
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  this_ptr->remove_higher_space_dimensions(ppl_dim);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_expand_1space_1dimension
(JNIEnv* env, jobject j_this, jobject j_variable, jlong dim) try {
  dimension_type ppl_dim = jtype_to_unsigned<dimension_type>(dim);
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_variable);
  this_ptr->expand_space_dimension(v, ppl_dim);
}
CATCH_ALL





JNIEXPORT void JNICALL Java_parma_1polyhedra_1library_Polyhedron_fold_1space_1dimensions
(JNIEnv* env, jobject j_this, jobject j_v_set, jobject j_var) try {
  Variables_Set v_set = build_cxx_variables_set(env, j_v_set);
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Variable v = build_cxx_variable(env, j_var);
  this_ptr->fold_space_dimensions(v_set, v);
}
CATCH_ALL





JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_map_1space_1dimensions
(JNIEnv* env, jobject j_this, jobject j_p_func) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Partial_Function ppl_pfunc(j_p_func, env);
  this_ptr->map_space_dimensions(ppl_pfunc);
}
CATCH_ALL





JNIEXPORT jstring JNICALL
Java_parma_1polyhedra_1library_Polyhedron_ascii_1dump
(JNIEnv* env, jobject j_this) {
  try {
    using namespace Parma_Polyhedra_Library::IO_Operators;
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    std::ostringstream s;
    this_ptr->ascii_dump(s);
    std::string str = s.str();
    return env->NewStringUTF(str.c_str());
  }
  CATCH_ALL;
  return 0;
}





JNIEXPORT jlong JNICALL
Java_parma_1polyhedra_1library_Polyhedron_external_1memory_1in_1bytes
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->external_memory_in_bytes();
  }
  CATCH_ALL;
  return 0;
}

JNIEXPORT jlong JNICALL
Java_parma_1polyhedra_1library_Polyhedron_total_1memory_1in_1bytes
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->total_memory_in_bytes();
  }
  CATCH_ALL;
  return 0;
}








JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_BHRZ03_1widening_1assign
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_by_ref_int) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  if (is_null(env, j_by_ref_int))
    this_ptr->BHRZ03_widening_assign(*y_ptr);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->BHRZ03_widening_assign(*y_ptr, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_H79_1widening_1assign
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_by_ref_int) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  if (is_null(env, j_by_ref_int))
    this_ptr->H79_widening_assign(*y_ptr);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->H79_widening_assign(*y_ptr, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL








JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_widening_1assign
(JNIEnv* env, jobject j_this, jobject j_y, jobject j_by_ref_int) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  if (is_null(env, j_by_ref_int))
    this_ptr->widening_assign(*y_ptr);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->widening_assign(*y_ptr, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL







JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_limited_1BHRZ03_1extrapolation_1assign
(JNIEnv* env, jobject j_this, jobject j_y,
 jobject j_cs, jobject j_by_ref_int) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  Constraint_System cs = build_cxx_constraint_system(env, j_cs);
  if (is_null(env, j_by_ref_int))
    this_ptr->limited_BHRZ03_extrapolation_assign(*y_ptr, cs);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->limited_BHRZ03_extrapolation_assign
      (*y_ptr, cs, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_bounded_1BHRZ03_1extrapolation_1assign
(JNIEnv* env, jobject j_this, jobject j_y,
 jobject j_cs, jobject j_by_ref_int) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  Constraint_System cs = build_cxx_constraint_system(env, j_cs);
  if (is_null(env, j_by_ref_int))
    this_ptr->bounded_BHRZ03_extrapolation_assign(*y_ptr, cs);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->bounded_BHRZ03_extrapolation_assign
      (*y_ptr, cs, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_limited_1H79_1extrapolation_1assign
(JNIEnv* env, jobject j_this, jobject j_y,
 jobject j_cs, jobject j_by_ref_int) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  Constraint_System cs = build_cxx_constraint_system(env, j_cs);
  if (is_null(env, j_by_ref_int))
    this_ptr->limited_H79_extrapolation_assign(*y_ptr, cs);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->limited_H79_extrapolation_assign
      (*y_ptr, cs, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_Polyhedron_bounded_1H79_1extrapolation_1assign
(JNIEnv* env, jobject j_this, jobject j_y,
 jobject j_cs, jobject j_by_ref_int) try {
  Polyhedron* this_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
  Polyhedron* y_ptr
    = reinterpret_cast<Polyhedron*>(get_ptr(env, j_y));
  Constraint_System cs = build_cxx_constraint_system(env, j_cs);
  if (is_null(env, j_by_ref_int))
    this_ptr->bounded_H79_extrapolation_assign(*y_ptr, cs);
  else {
    jobject j_integer = get_by_reference(env, j_by_ref_int);
    unsigned int tokens =
      jtype_to_unsigned<unsigned int>(j_integer_to_j_int(env, j_integer));
    this_ptr->bounded_H79_extrapolation_assign
      (*y_ptr, cs, &tokens);
    j_integer = j_int_to_j_integer(env, tokens);
    set_by_reference(env, j_by_ref_int, j_integer);
  }
}
CATCH_ALL








JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_linear_1partition
(JNIEnv* env, jclass, jobject j_p, jobject j_q) {
  try {
    C_Polyhedron* p_ptr
      = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_p));
    C_Polyhedron* q_ptr
      = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_q));
    std::pair<C_Polyhedron,
              Pointset_Powerset<NNC_Polyhedron> >
      r = linear_partition(*p_ptr, *q_ptr);

    C_Polyhedron* r1
      = new C_Polyhedron(0, EMPTY);
    Pointset_Powerset<NNC_Polyhedron>* r2
      = new Pointset_Powerset<NNC_Polyhedron>(0, EMPTY);
    r1->swap(r.first);
    r2->swap(r.second);

    jclass j_pair_class = env->FindClass("parma_polyhedra_library/Pair");
    jmethodID j_ctr_id_pair = env->GetMethodID(j_pair_class, "<init>", "()V");
    jobject j_pair_obj = env->NewObject(j_pair_class, j_ctr_id_pair);

    jclass j_class_r1 = env->FindClass("parma_polyhedra_library/C_Polyhedron");
    jmethodID j_ctr_id_r1 = env->GetMethodID(j_class_r1, "<init>", "()V");
    jobject j_obj_r1 = env->NewObject(j_class_r1, j_ctr_id_r1);
    set_ptr(env, j_obj_r1, r1);

    jclass j_class_r2 = env->FindClass("parma_polyhedra_library/Pointset_Powerset_NNC_Polyhedron");
    jmethodID j_ctr_id_r2 = env->GetMethodID(j_class_r2, "<init>", "()V");
    jobject j_obj_r2 = env->NewObject(j_class_r2, j_ctr_id_r2);
    set_ptr(env, j_obj_r2, r2);
    set_pair_element(env, j_pair_obj, 0, j_obj_r1);
    set_pair_element(env, j_pair_obj, 1, j_obj_r2);
    return j_pair_obj;
  }
  CATCH_ALL;
  return 0;
}

JNIEXPORT jobject JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_linear_1partition
(JNIEnv* env, jclass, jobject j_p, jobject j_q) {
  try {
    NNC_Polyhedron* p_ptr
      = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_p));
    NNC_Polyhedron* q_ptr
      = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_q));
    std::pair<NNC_Polyhedron,
              Pointset_Powerset<NNC_Polyhedron> >
      r = linear_partition(*p_ptr, *q_ptr);

    NNC_Polyhedron* r1
      = new NNC_Polyhedron(0, EMPTY);
    Pointset_Powerset<NNC_Polyhedron>* r2
      = new Pointset_Powerset<NNC_Polyhedron>(0, EMPTY);
    r1->swap(r.first);
    r2->swap(r.second);

    jclass j_pair_class = env->FindClass("parma_polyhedra_library/Pair");
    jmethodID j_ctr_id_pair = env->GetMethodID(j_pair_class, "<init>", "()V");
    jobject j_pair_obj = env->NewObject(j_pair_class, j_ctr_id_pair);

    jclass j_class_r1 = env->FindClass("parma_polyhedra_library/NNC_Polyhedron");
    jmethodID j_ctr_id_r1 = env->GetMethodID(j_class_r1, "<init>", "()V");
    jobject j_obj_r1 = env->NewObject(j_class_r1, j_ctr_id_r1);
    set_ptr(env, j_obj_r1, r1);

    jclass j_class_r2 = env->FindClass("parma_polyhedra_library/Pointset_Powerset_NNC_Polyhedron");
    jmethodID j_ctr_id_r2 = env->GetMethodID(j_class_r2, "<init>", "()V");
    jobject j_obj_r2 = env->NewObject(j_class_r2, j_ctr_id_r2);
    set_ptr(env, j_obj_r2, r2);
    set_pair_element(env, j_pair_obj, 0, j_obj_r1);
    set_pair_element(env, j_pair_obj, 1, j_obj_r2);
    return j_pair_obj;
  }
  CATCH_ALL;
  return 0;
}







JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_C_1Polyhedron_free
(JNIEnv* env, jobject j_this) try {
  if (!is_java_marked(env, j_this)) {
    C_Polyhedron* this_ptr
      = reinterpret_cast<C_Polyhedron*>(get_ptr(env, j_this));
    delete this_ptr;
    void* null_ptr = 0;
    set_ptr(env, j_this, null_ptr);
  }
}
CATCH_ALL

JNIEXPORT void JNICALL
Java_parma_1polyhedra_1library_NNC_1Polyhedron_free
(JNIEnv* env, jobject j_this) try {
  if (!is_java_marked(env, j_this)) {
    NNC_Polyhedron* this_ptr
      = reinterpret_cast<NNC_Polyhedron*>(get_ptr(env, j_this));
    delete this_ptr;
    void* null_ptr = 0;
    set_ptr(env, j_this, null_ptr);
  }
}
CATCH_ALL






JNIEXPORT jint JNICALL
Java_parma_1polyhedra_1library_Polyhedron_hashCode
(JNIEnv* env, jobject j_this) {
  try {
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    return this_ptr->hash_code();
  }
  CATCH_ALL;
  return 0;
}





JNIEXPORT jstring JNICALL
Java_parma_1polyhedra_1library_Polyhedron_toString
(JNIEnv* env, jobject j_this) {
  try {
    using namespace Parma_Polyhedra_Library::IO_Operators;
    Polyhedron* this_ptr
      = reinterpret_cast<Polyhedron*>(get_ptr(env, j_this));
    std::ostringstream s;
    s << *this_ptr;
    std::string str = s.str();
    return env->NewStringUTF(str.c_str());
  }
  CATCH_ALL;
  return 0;
}







